//********************************************************************
UNIT GUIBase;
//********************************************************************
INTERFACE

USES SysUtils, Contnrs, GraphicSupport;
//********************************************************************
TYPE
   TGUIClassName = String;
   TGUIIdentity = String;
   TGUIColorName = String;
   TGUIColorData = Record ColName: String; RGBAColor: TRGBAColor; End;
   TGUIPosition = Longint;
   TGUIPositionRect = Record
      Left,Top,Right,Bottom: TGUIPosition;
   End;
   TGUIVisible = Boolean;
   TGUIThemesIndex = LongInt;
//********************************************************************
   TGUIDataProvider = Class(TObject)
   // Open/Close
      Function OpenSource(ASourceName: String): Boolean; Virtual; Abstract;
      Procedure CloseSource; Virtual; Abstract;

   // Data loaded by Themes
      Function LoadCount: Integer; Virtual; Abstract;
      Procedure SaveCount(ACount: Integer); Virtual; Abstract;
      Function LoadColorData: TGUIColorData; Virtual; Abstract;
      Procedure SaveColorData(AColorData: TGUIColorData); Virtual; Abstract;

   // Data loaded by TGUIItems + Descendents
      Function LoadClassName: TGUIClassName; Virtual; Abstract;
      Procedure SaveClassName(AClassName: TGUIClassName); Virtual; Abstract;
      Function LoadIdentity: TGUIIdentity; Virtual; Abstract;
      Procedure SaveIdentity(AIdentity: TGUIIdentity); Virtual; Abstract;
      Function LoadPositionRect: TGUIPositionRect; Virtual; Abstract;
      Procedure SavePositionRect(APositionRect: TGUIPositionRect); Virtual; Abstract;
      Function LoadVisible: TGUIVisible; Virtual; Abstract;
      Procedure SaveVisible(AVisible: TGUIVisible); Virtual; Abstract;
      Function LoadColorName: TGUIColorName; Virtual; Abstract;
      Procedure SaveColorName(AColorName: TGUIColorName); Virtual; Abstract;
   End;
//********************************************************************
// Visible Objects
{$M+}
   TGUIVisibleObject = Class(TObject)
   Private
      FName: String;
   Published
      Property Name: String Read FName Write FName;
   End;
{$M-}
//********************************************************************
   TGUIColor = Class(TGUIVisibleObject)
   Private
      FBaseColor: TRGBAColor;
      FShownColor: TRGBAColor;

       Function GetRGBAColor: TRGBAColor;
      Procedure SetRGBAColor(ARGBAColor: TRGBAColor);
   Public
      Property Color: TRGBAColor Read GetRGBAColor Write SetRGBAColor;
      Procedure Load(ADataProvider: TGUIDataProvider);
      Procedure Save(ADataProvider: TGUIDataProvider);
      Procedure Reset;
      Procedure SetLuminance(ALuminance: Single);
   End;
//********************************************************************
   TThemes = Class(TObject)
   Private
      FColor: TGUIColor;
      FLineWidth: Integer;

      FColors: TObjectList;
      FDataProvider: TGUIDataProvider;

       Function GetColor: TGUIColor;
      Procedure SetColor(AColor: TGUIColor);
       Function GetLineWidth: Integer;
      Procedure SetLineWidth(ALineWidth: Integer);
   Public
      Property Color: TGUIColor
         Read GetColor Write SetColor;
      Property LineWidth: Integer
         Read GetLineWidth Write SetLineWidth;
      Property DataProvider: TGUIDataProvider
         Write FDataProvider;

      Constructor Create; Reintroduce;
      Destructor Destroy; Override;

      // Administrate colors
      Function ActivateColor(AColorIndex: TGUIThemesIndex): Boolean;
      Procedure AddColor(AColorName: String; ARGBAColor: TRGBAColor);
      Function ColorIndex(AColorName: TGUIColorName): TGUIThemesIndex;
      Procedure LoadColors;
      Procedure SaveColors;

      // Parameters = ALeft,ATop,ARight,ABottom ----------------------
      Procedure Ellipse(ALeft,ATop,ARight,ABottom: Integer;
                                       Fill: Boolean);
      Procedure Line(FromX,FromY,ToX,ToY: Integer);
      Procedure Rectangle(ALeft,ATop,ARight,ABottom: Integer;
                                       Fill: Boolean);
      Procedure RoundRect(ALeft,ATop,ARight,ABottom: Integer;
                                       Fill: Boolean);

      // Parameters = Array Of TPoint2D ------------------------------
      Procedure Graph(APoint: Array Of TPoint2D);
      Procedure Polygon(APoint: Array Of TPoint2D; Fill: Boolean);
   End;
//********************************************************************
   TMouseImage = (miArrow,miCaret);
//********************************************************************
   TMouse = Class(TObject)
   Private
      FMouseImage: TMouseImage;
   Public
      Constructor Create; Reintroduce;
      Destructor Destroy; Override;
      Procedure Draw;
       Function GetMouseImage: TMouseImage;
      Procedure SetMouseImage(AMouseImage: TMouseImage);
   End;
//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
//   T G U I C O L O R
//********************************************************************
//
Function TGUIColor.GetRGBAColor: TRGBAColor;
Begin
   Result:= FShownColor;
End;
//********************************************************************
Procedure TGUIColor.SetRGBAColor(ARGBAColor: TRGBAColor);
Begin
   FBaseColor:= ARGBAColor;
   FShownColor:= FBaseColor;
End;
//********************************************************************
Procedure TGUIColor.Load(ADataProvider: TGUIDataProvider);
Var ColorData: TGUIColorData;
Begin
   ColorData:= ADataProvider.LoadColorData;
   With ColorData Do Begin
      Name:= ColName;
      FBaseColor:= RGBAColor;
   End;
End;
//********************************************************************
Procedure TGUIColor.Save(ADataProvider: TGUIDataProvider);
Var ColorData: TGUIColorData;
Begin
   With ColorData Do Begin
      ColName:= Name;
      RGBAColor:= FBaseColor;
   End;
   ADataProvider.SaveColorData(ColorData);
End;
//********************************************************************
// Sets the shown color back
Procedure TGUIColor.Reset;
Begin
   FShownColor:= FBaseColor;
End;
//********************************************************************
// Set the luminance of the shown color
Procedure TGUIColor.SetLuminance(ALuminance: Single);
Begin
   If (ALuminance >= 0) And (ALuminance <= 1)
      Then With FShownColor Do Begin
           Red:= FBaseColor.Red * ALuminance;
         Green:= FBaseColor.Green * ALuminance;
          Blue:= FBaseColor.Blue * ALuminance;
      End;
End;
//********************************************************************
//********************************************************************
//   T T H E M E S
//********************************************************************
// No comment
Constructor TThemes.Create;
Begin
   Inherited Create;
   FColors:= TObjectList.Create;

   AddColor('SOLIDWHITE', SolidWhite);
   AddColor('SOLIDBLACK', SolidBlack);
   AddColor('SOLIDGREY', SolidGrey);

   AddColor('SOLIDRED', SolidRed);
   AddColor('SOLIDGREEN', SolidGreen);
   AddColor('SOLIDBLUE', SolidBlue);

   AddColor('SOLIDYELLOW', SolidYellow);
   AddColor('SOLIDCYAN', SolidCyan);
   AddColor('SOLIDMAGENTA', SolidMagenta);
   AddColor('SOLIDORANGE', SolidOrange);   
End;
//********************************************************************
// No comment
Destructor TThemes.Destroy;
Begin
   FColors.Free;
   Inherited Destroy;
End;
//********************************************************************
// No comment
Function TThemes.GetColor: TGUIColor;
Begin Result:= FColor; End;
//********************************************************************
// No comment
Procedure TThemes.SetColor(AColor: TGUIColor);
Begin FColor:= AColor; End;
//********************************************************************
// No comment
Function TThemes.GetLineWidth: Integer;
Begin Result:= FLineWidth; End;
//********************************************************************
// No comment
Procedure TThemes.SetLineWidth(ALineWidth: Integer);
Begin FLineWidth:= ALineWidth; End;
//********************************************************************
//   T T H E M E S   -   Administrate colors
//********************************************************************
Function TThemes.ActivateColor(AColorIndex: TGUIThemesIndex): Boolean;
Begin
   Result:= False;
   With FColors Do
      If (Count > 0) And
         (AColorIndex >= 0) And
         (AColorIndex < Count)
         Then Begin
            FColor:= List[AColorIndex];
            Result:= True;
         End;
End;
//********************************************************************
Procedure TThemes.AddColor(AColorName: String; ARGBAColor: TRGBAColor);
Var LocalColor: TGUIColor;
Begin
   LocalColor:= TGUIColor.Create;
   With LocalColor Do Begin
      Name:= AColorName;
      Color:= ARGBAColor;
   End;
   FColors.Add(LocalColor);
End;
//********************************************************************
// The names of the listed colors are supposed to be upper case
Function TThemes.ColorIndex(AColorName: TGUIColorName): TGUIThemesIndex;
Var LocalColor: TGUIColor; I: Integer;
Begin
   Result:= -1;
   AColorName:= UpperCase(AColorName);
   I:= 0;
   With FColors Do If Count > 0
      Then While (I < Count) And (Result < 0) Do Begin
         LocalColor:= List[I];
         If LocalColor.Name <> AColorName
            Then Inc(I) //=================>
            Else Result:= I;
      End;
End;
//********************************************************************
Procedure TThemes.LoadColors;
Var LocalColor: TGUIColor; ColorCount, I: Integer;
Begin
   ColorCount:= FDataProvider.LoadCount;
   For I:= 0 To ColorCount-1 Do Begin
      LocalColor:= TGUIColor.Create;
      LocalColor.Load(FDataProvider);
      FColors.Add(LocalColor);
   End;
End;
//********************************************************************
Procedure TThemes.SaveColors;
Var LocalColor: TGUIColor; I: Integer;
Begin
   With FColors Do Begin
      FDataProvider.SaveCount(Count);
      For I:= 0 To Count-1 Do Begin
         LocalColor:= List[I];
         LocalColor.Save(FDataProvider);
      End;
   End;
End;
//********************************************************************
//   T T H E M E S   -   Draw (ALeft,ATop,ARight,ABottom)
//********************************************************************
// Draws an ellipse
Procedure TThemes.Ellipse(ALeft,ATop,ARight,ABottom: Integer;
                                             Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Ellipse2D(ALeft,ATop,ARight,ABottom,Fill);
End;
//********************************************************************
// Draws a 2D-line from startpoint to endpoint
Procedure TThemes.Line(FromX,FromY,ToX,ToY: Integer);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Line2D(FromX,FromY,ToX,ToY);
End;
//********************************************************************
// Draws a Rectangle with rounded corners
Procedure TThemes.Rectangle(ALeft,ATop,ARight,ABottom: Integer;
                                  Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Rectangle2D(ALeft,ATop,ARight,ABottom,Fill);
End;
//********************************************************************
// Draws a Rectangle with rounded corners
Procedure TThemes.RoundRect(ALeft,ATop,ARight,ABottom: Integer;
                                  Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_RoundRect2D(ALeft,ATop,ARight,ABottom,Fill);
End;
//********************************************************************
//   T T H E M E S  -   Draw (Array Of TPoint2D)
//********************************************************************
// Graph plotter
Procedure TThemes.Graph(APoint: Array Of TPoint2D);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Graph2D(APoint);
End;
//********************************************************************
// Draws a polygon either with lines only or filled with current color
Procedure TThemes.Polygon(APoint: Array Of TPoint2D;
                                 Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Polygon2D(APoint,Fill);
End;
//********************************************************************
//********************************************************************
//   T M O U S E
//********************************************************************
Constructor TMouse.Create;
Begin
   Inherited Create;
   FMouseImage:= miArrow;
End;
//********************************************************************
Destructor TMouse.Destroy;
Begin
   Inherited Destroy;
End;
//********************************************************************
Procedure TMouse.Draw;
Begin
End;
//********************************************************************
Function TMouse.GetMouseImage: TMouseImage;
Begin Result:= FMouseImage; End;
//********************************************************************
Procedure TMouse.SetMouseImage(AMouseImage: TMouseImage);
Begin FMouseImage:= AMouseImage; End;
//********************************************************************
//********************************************************************
End.
